Package: timecourse
Version: 1.83.0
Date: 2007-6-3
Title: Statistical Analysis for Developmental Microarray Time Course
        Data
Author: Yu Chuan Tai 
Maintainer: Yu Chuan Tai <yuchuan@stat.berkeley.edu>
Depends: R (>= 2.1.1), MASS, methods
Imports: Biobase, graphics, limma (>= 1.8.6), MASS, marray, methods,
        stats
Description: Functions for data analysis and graphical displays for developmental microarray time course data.
License: LGPL
URL: http://www.bioconductor.org
biocViews: Microarray, TimeCourse, DifferentialExpression
git_url: https://git.bioconductor.org/packages/timecourse
git_branch: devel
git_last_commit: 7b6e6b3
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:26:25 UTC; biocbuild
